@@ -159,6 +159,22 @@ class InTest(unittest.TestCase):
     self.assert_(mox.In("test") == {"test" : "module"})
 
 
+class NotTest(unittest.TestCase):
+  """Test Not correctly identifies False predicates."""
+
+  def testItemInList(self):
+    """Should return True if the item is NOT in the list."""
+    self.assert_(mox.Not(mox.In(42)) == [1, 2, 3])
+
+  def testKeyInDict(self):
+    """Should return True if the item is NOT a key in a dict."""
+    self.assert_(mox.Not(mox.In("foo")) == {"key" : 42})
+
+  def testInvalidKeyWithNot(self):
+    """Should return False if they key is NOT in the dict."""
+    self.assert_(mox.Not(mox.ContainsKeyValue("qux", 1)) == {"key": 2})
+
+
 class StrContainsTest(unittest.TestCase):
   """Test StrContains correctly checks for substring occurrence of a parameter.
   """
@@ -517,6 +533,148 @@ class MockAnythingTest(unittest.TestCase):
 
     self.mock_object._Verify()
 
+  def testIsReprable(self):
+    """Test that MockAnythings can be repr'd without causing a failure."""
+    self.failUnless('MockAnything' in repr(self.mock_object))
+
+
+class MethodCheckerTest(unittest.TestCase):
+  """Tests MockMethod's use of MethodChecker method."""
+
+  def testNoParameters(self):
+    method = mox.MockMethod('NoParameters', [], False,
+                            CheckCallTestClass.NoParameters)
+    method()
+    self.assertRaises(AttributeError, method, 1)
+    self.assertRaises(AttributeError, method, 1, 2)
+    self.assertRaises(AttributeError, method, a=1)
+    self.assertRaises(AttributeError, method, 1, b=2)
+
+  def testOneParameter(self):
+    method = mox.MockMethod('OneParameter', [], False,
+                            CheckCallTestClass.OneParameter)
+    self.assertRaises(AttributeError, method)
+    method(1)
+    method(a=1)
+    self.assertRaises(AttributeError, method, b=1)
+    self.assertRaises(AttributeError, method, 1, 2)
+    self.assertRaises(AttributeError, method, 1, a=2)
+    self.assertRaises(AttributeError, method, 1, b=2)
+
+  def testTwoParameters(self):
+    method = mox.MockMethod('TwoParameters', [], False,
+                            CheckCallTestClass.TwoParameters)
+    self.assertRaises(AttributeError, method)
+    self.assertRaises(AttributeError, method, 1)
+    self.assertRaises(AttributeError, method, a=1)
+    self.assertRaises(AttributeError, method, b=1)
+    method(1, 2)
+    method(1, b=2)
+    method(a=1, b=2)
+    method(b=2, a=1)
+    self.assertRaises(AttributeError, method, b=2, c=3)
+    self.assertRaises(AttributeError, method, a=1, b=2, c=3)
+    self.assertRaises(AttributeError, method, 1, 2, 3)
+    self.assertRaises(AttributeError, method, 1, 2, 3, 4)
+    self.assertRaises(AttributeError, method, 3, a=1, b=2)
+
+  def testOneDefaultValue(self):
+    method = mox.MockMethod('OneDefaultValue', [], False,
+                            CheckCallTestClass.OneDefaultValue)
+    method()
+    method(1)
+    method(a=1)
+    self.assertRaises(AttributeError, method, b=1)
+    self.assertRaises(AttributeError, method, 1, 2)
+    self.assertRaises(AttributeError, method, 1, a=2)
+    self.assertRaises(AttributeError, method, 1, b=2)
+
+  def testTwoDefaultValues(self):
+    method = mox.MockMethod('TwoDefaultValues', [], False,
+                            CheckCallTestClass.TwoDefaultValues)
+    self.assertRaises(AttributeError, method)
+    self.assertRaises(AttributeError, method, c=3)
+    self.assertRaises(AttributeError, method, 1)
+    self.assertRaises(AttributeError, method, 1, d=4)
+    self.assertRaises(AttributeError, method, 1, d=4, c=3)
+    method(1, 2)
+    method(a=1, b=2)
+    method(1, 2, 3)
+    method(1, 2, 3, 4)
+    method(1, 2, c=3)
+    method(1, 2, c=3, d=4)
+    method(1, 2, d=4, c=3)
+    method(d=4, c=3, a=1, b=2)
+    self.assertRaises(AttributeError, method, 1, 2, 3, 4, 5)
+    self.assertRaises(AttributeError, method, 1, 2, e=9)
+    self.assertRaises(AttributeError, method, a=1, b=2, e=9)
+
+  def testArgs(self):
+    method = mox.MockMethod('Args', [], False, CheckCallTestClass.Args)
+    self.assertRaises(AttributeError, method)
+    self.assertRaises(AttributeError, method, 1)
+    method(1, 2)
+    method(a=1, b=2)
+    method(1, 2, 3)
+    method(1, 2, 3, 4)
+    self.assertRaises(AttributeError, method, 1, 2, a=3)
+    self.assertRaises(AttributeError, method, 1, 2, c=3)
+
+  def testKwargs(self):
+    method = mox.MockMethod('Kwargs', [], False, CheckCallTestClass.Kwargs)
+    self.assertRaises(AttributeError, method)
+    method(1)
+    method(1, 2)
+    method(a=1, b=2)
+    method(b=2, a=1)
+    self.assertRaises(AttributeError, method, 1, 2, 3)
+    self.assertRaises(AttributeError, method, 1, 2, a=3)
+    method(1, 2, c=3)
+    method(a=1, b=2, c=3)
+    method(c=3, a=1, b=2)
+    method(a=1, b=2, c=3, d=4)
+    self.assertRaises(AttributeError, method, 1, 2, 3, 4)
+
+  def testArgsAndKwargs(self):
+    method = mox.MockMethod('ArgsAndKwargs', [], False,
+                            CheckCallTestClass.ArgsAndKwargs)
+    self.assertRaises(AttributeError, method)
+    method(1)
+    method(1, 2)
+    method(1, 2, 3)
+    method(a=1)
+    method(1, b=2)
+    self.assertRaises(AttributeError, method, 1, a=2)
+    method(b=2, a=1)
+    method(c=3, b=2, a=1)
+    method(1, 2, c=3)
+
+
+class CheckCallTestClass(object):
+  def NoParameters(self):
+    pass
+
+  def OneParameter(self, a):
+    pass
+
+  def TwoParameters(self, a, b):
+    pass
+
+  def OneDefaultValue(self, a=1):
+    pass
+
+  def TwoDefaultValues(self, a, b, c=1, d=2):
+    pass
+
+  def Args(self, a, b, *args):
+    pass
+
+  def Kwargs(self, a, b=2, **kwargs):
+    pass
+
+  def ArgsAndKwargs(self, a, *args, **kwargs):
+    pass
+
 
 class MockObjectTest(unittest.TestCase):
   """Verify that the MockObject class works as exepcted."""
@@ -732,6 +890,46 @@ class MockObjectTest(unittest.TestCase):
 
     dummy._Verify()
 
+  def testMockContains_ExpectedContains_Success(self):
+    """Test that __contains__ gets mocked in Dummy.
+
+    In this test, _Verify() succeeds.
+    """
+    dummy = mox.MockObject(TestClass)
+    dummy.__contains__('X').AndReturn(True)
+
+    dummy._Replay()
+
+    self.failUnless('X' in dummy)
+
+    dummy._Verify()
+
+  def testMockContains_ExpectedContains_NoSuccess(self):
+    """Test that __contains__() gets mocked in Dummy.
+
+    In this test, _Verify() fails.
+    """
+    dummy = mox.MockObject(TestClass)
+    dummy.__contains__('X').AndReturn('True')
+
+    dummy._Replay()
+
+    # NOT doing 'X' in dummy
+
+    self.assertRaises(mox.ExpectedMethodCallsError, dummy._Verify)
+
+  def testMockContains_ExpectedContains_NonmatchingParameter(self):
+    """Test that __contains__ fails if other parameters are expected."""
+    dummy = mox.MockObject(TestClass)
+    dummy.__contains__('X').AndReturn(True)
+
+    dummy._Replay()
+
+    def call(): return 'Y' in dummy
+
+    self.assertRaises(mox.UnexpectedMethodCallError, call)
+
+    dummy._Verify()
 
 class MoxTest(unittest.TestCase):
   """Verify Mox works correctly."""
@@ -1250,6 +1448,26 @@ class ResetTest(unittest.TestCase):
     self.assertEquals(0, len(mock_obj._expected_calls_queue))
 
 
+class MyTestCase(unittest.TestCase):
+  """Simulate the use of a fake wrapper around Python's unittest library."""
+
+  def setUp(self):
+    super(MyTestCase, self).setUp()
+    self.critical_variable = 42
+
+
+class MoxTestBaseMultipleInheritanceTest(mox.MoxTestBase, MyTestCase):
+  """Test that multiple inheritance can be used with MoxTestBase."""
+
+  def setUp(self):
+    super(MoxTestBaseMultipleInheritanceTest, self).setUp()
+
+  def testMultipleInheritance(self):
+    """Should be able to access members created by all parent setUp()."""
+    self.assert_(isinstance(self.mox, mox.Mox))
+    self.assertEquals(42, self.critical_variable)
+
+
 class TestClass:
   """This class is used only for testing the mock framework"""
 
@@ -1302,6 +1520,10 @@ class TestClass:
     """Set the value for key to value."""
     self.d[key] = value
 
+  def __contains__(self, key):
+     """Returns True if d contains the key."""
+     return key in self.d
+
 
 class ChildClass(TestClass):
   """This inherits from TestClass."""
